<?php

namespace App\Http\Controllers\Admin;

use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\StoreUsersRequest;
use App\Http\Requests\Admin\UpdateUsersRequest;

use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use DB;
use Hash;
use DataTables;

use App\Imports\UsersImport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Arr;

use Brian2694\Toastr\Facades\Toastr;

class UsersController extends Controller
{
    /**
     * Display a listing of User.
     *
     * @return \Illuminate\Http\Response
     */

    function __construct()
    {
         
         //$this->middleware('permission:Leaves-list|Leaves-show|Leaves-create|Leaves-edit|Leaves-delete|Leaves-import', ['only' => ['index','store']]);
         $this->middleware('permission:Users-list|Users-show', ['only' => ['index','show']]); 
         $this->middleware('permission:Users-list', ['only' => ['show']]);
         $this->middleware('permission:Users-create', ['only' => ['create','store']]);
         $this->middleware('permission:Users-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:Users-delete', ['only' => ['destroy']]);
         $this->middleware('permission:Users-import', ['only' => ['importExportView']]);
         $this->middleware('permission:Users-export', ['only' => ['export']]);
    }

    public function index(Request $request)
    {

         if(request()->ajax())
         {
              if(!empty($request->name) || !empty($request->email) || !empty($request->status))
              {

                $query = User::whereHas('roles', function ($query) {
                    $query->where('roles.name','!=','Backup');
                });


                if(!empty($request->name)) {
                    $query = $query->where('users.name', 'LIKE',"%{$request->name}%");
                }  
                if(!empty($request->email)) {
                    $query = $query->where('users.email', 'LIKE',"%{$request->email}%");
                }
                if(!empty($request->status)) {
                    $query = $query->where('users.status', '=', $request->status);
                }                 
                //->where('users.name', 'LIKE',"%{$request->sname}%")
                 $query = $query->with('roles');
                 $query = $query->get();  
                $data= $query;
              }
              else
              { 
               //$data = DB::table('disciplinaries')
               //  ->select('*');
                 //->get();
                $data = User::whereHas('roles', function ($query) {
                    $query->where('roles.name', '!=', 'Backup');
                })
                ->with('roles')
                ->get();               
              }
                $data = $data->transform(function ($item) {
                    $item->role_names = $item->roles->pluck('name')->implode('|');
                    //$item->role_names = $item->roles->pluck('name')->implode(', ');
                    return $item;
                })->all();


                //return DataTables::collection($data)->toJson()


             return datatables()->of($data) 
                    
            ->addColumn('action', function ($row) {
                $show = route('admin.users.show',$row->id);
                $edit =  route('admin.users.edit',$row->id);
                $delete =  route('admin.users.destroy',$row->id);
                $token =  csrf_token();                  
                //return "<a href='{$show}' class='btn btn-xs btn-primary'><i class='glyphicon glyphicon-edit'></i> Edit</a>";

                return "<div class='btn-group'>
                            <button aria-expanded='false' aria-haspopup='true' data-toggle='dropdown' data-boundary='viewport' class='btn-xs btn-primary dropdown-toggle' type='button'>
                            <small>Action</small> 
                            <span class='caret'></span></button>

                            <div class='dropdown-menu' style='font-size:12px'>
                              <a class='dropdown-item' href='{$show}'><i class='fa fa-desktop text-blue' style='color:blue;'></i><small> View</small></a>
                              <a class='dropdown-item' href='{$edit}'><i class='fa fa-edit text-green' style='color:green;'></i><small> Edit</small></a>
                              <form action='{$delete}' method='POST' onsubmit=\'return confirm('Are you sure you want to Delete Selected Record?');\' style='display: inline-block;'>".
                               "<input type='hidden' name='_method' value='DELETE'>".
                               "<input type='hidden' name='_token' value='{$token}'>". 
                               "<button type='submit' title = 'Delete' class='dropdown-item' style='border:none; background:none; cursor:pointer;'><i class='fa fa-trash text-red' style='color:red;'></i><small> Delete</small></button>".
                               "</form>
                            </div>
                        </div>";
                
            })
            ->make(true);


             // return datatables()->of($data)->make(true);
    


         }

         $users = User::whereHas('roles', function ($query) {
                    $query->where('roles.name', '!=', 'Backup');
                })
                ->with('roles')
                ->get();   

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('admin.users.index', compact('users'));
         } else {
            return view('admin.users.indexm', compact('users'));
         }                
              
         
    }

    /**
     * Show the form for creating new User.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {

        $roles = Role::get()->pluck('name', 'name');

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('admin.users.create', compact('roles'));
         } else {
            return view('admin.users.createm', compact('roles'));
         }          

        
    }

    /**
     * Store a newly created User in storage.
     *
     * @param  \App\Http\Requests\StoreUsersRequest  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, User $users)
    {

        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:users,email',
            'password' => 'required|same:confirm-password',
            'roles' => 'required'
        ]);
        $input = $request->all();
        $users->name = $request->name;

        if ($request->hasFile('emp_photo')){
            if($request->emp_photo->getClientOriginalName())
            {
                $ext = $request->emp_photo->getClientOriginalExtension();
                $emp_photo = $request->name. '.'.$ext;
                $request->emp_photo->storeAs('public/employees', $emp_photo);
            }else{
                $emp_photo='user.png';            
            }
        } else {
           $emp_photo='user.png';        
        }       
    
        
        //$input['password'] = Hash::make($input['password']);

        $users->name = $request->name;
        $users->filename = $emp_photo;
        $users->email = $request->email;
        $users->password = Hash::make($request->password);
        $users->save();        




        
        //$user = User::create($input);
        $users->assignRole($request->input('roles'));


        Toastr::success('User Added Successfully :)','User Management');

        return redirect()->route('admin.users.index')
                        ->with('success','User created successfully');
                        
    }


    /**
     * Show the form for editing User.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(User $user)
    {

        $roles = Role::get()->pluck('name', 'name');

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('admin.users.edit', compact('user', 'roles'));
         } else {
            return view('admin.users.editm', compact('user', 'roles'));
         }  


        
    }

    /**
     * Update User in storage.
     *
     * @param  \App\Http\Requests\UpdateUsersRequest  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:users,email,'.$id,
            'password' => 'same:confirm-password',
            'roles' => 'required'
        ]);

        $users = User::where('id', '=', $id)->first(); 

        $users->name = $request->name;
        if ($request->hasFile('emp_photo')){
            if($request->emp_photo->getClientOriginalName())
            {
                $ext = $request->emp_photo->getClientOriginalExtension();
                $emp_photo = $request->name.'.'.$ext;
                $request->emp_photo->storeAs('public/employees', $emp_photo);
            }else{
                $emp_photo='';            
            }
        } else {
            $emp_photo='';        
        }
        if ($request->hasFile('emp_photo')){
            $users->filename = $emp_photo;    
        } 


        $input = $request->all();
        if(!empty($input['password'])){ 
            $input['password'] = Hash::make($input['password']);
        }else{
            $input = Arr::except($input,array('password'));    
        }

        $users->email = $request->email;

        if($request->password != ""){
            $users->password = Hash::make($request->password);
        }
        $users->save();        
    
        //$user = User::find($id);
        //$user->update($input);
        DB::table('model_has_roles')->where('model_id',$id)->delete();
    
        $users->assignRole($request->input('roles'));
        

        Toastr::success('User updated Successfully :)','User Management');

        return redirect()->route('admin.users.index')
                        ->with('success','User updated successfully');
    }

    public function show(User $user)
    {


        $roles = Role::get()->pluck('name', 'name');

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('admin.users.show', compact('user', 'roles'));
         } else {
            return view('admin.users.showm', compact('user', 'roles'));
         }  


        
    }

    /**
     * Remove User from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(User $user)
    {
        $user->delete();
        Toastr::warning('User deleted Successfully :)','User Management');
        return redirect()->route('admin.users.index');
    }

    /**
     * Delete all selected User at once.
     *
     * @param Request $request
     */
    public function massDestroy(Request $request)
    {

        User::whereIn('id', request('ids'))->delete();

        return response()->noContent();
    }


   public function allUsers(Request $request)
    {
        
        $columns = array( 
                            0 =>'id', 
                            1 =>'name',
                            2=> 'email',
                            3=> 'name',
                        );
  
        $totalData = User::count();
            
        $totalFiltered = $totalData; 

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
            
        if(empty($request->input('search.value')))
        {            
            $posts = User::offset($start)
                         ->limit($limit)
                         //->orderBy($order,$dir)
                         ->orderBy('name',"ASC")
                         ->get();
        }
        else {
            $search = $request->input('search.value'); 

            $posts =  User::where('id','LIKE',"%{$search}%")
                            ->orWhere('name', 'LIKE',"%{$search}%")
                            ->orWhere('email', 'LIKE',"%{$search}%")
                            ->orWhere('name', 'LIKE',"%{$search}%")
                            ->offset($start)
                            ->limit($limit)
                            ->orderBy($order,$dir)
                            ->get();

            $totalFiltered = User::where('id','LIKE',"%{$search}%")
                             ->orWhere('name', 'LIKE',"%{$search}%")
                             ->orWhere('email', 'LIKE',"%{$search}%")
                             ->orWhere('name', 'LIKE',"%{$search}%")
                             ->count();
        }

        $data = array();
        if(!empty($posts))
        {
            foreach ($posts as $post)
            {
                $show =  route('admin.users.show',$post->id);
                $edit =  route('admin.users.edit',$post->id);
                $delete =  route('admin.users.destroy',$post->id);
                $token =  csrf_token();

                   
                  $roles = "";
                  foreach($post->roles()->pluck('name') as $role){
                        $roles = $roles. '<span class="badge badge-info">'.$role. '</span>';   
                   }; 

                   $row[6] = $roles;
                                          


                $nestedData['id'] = $post->id;
                $nestedData['name'] = $post->name;
                $nestedData['email'] = $post->email;
                //foreach ($post->roles()->pluck('name') as $role){
                //    $nestedData['role'] = $role;
                //}
                $nestedData['role'] = $row[6];


                $nestedData['options'] = "<a href='{$show}' title='View' class='m-r-15 crm-action-edit text-muted'><i class='fa fa-desktop'></i></a>  &nbsp;<a href='{$edit}' title='Edit' class='m-r-15 crm-action-edit text-muted'><i class='fa fa-edit'></i></a> ". 
                    "<form action='{$delete}' name='deleteform' method='POST' onsubmit=\"return confirm('Are you sure you want to Delete Selected Record?');\" style='display: inline-block;'>".
                        "<input type='hidden' name='_method' value='DELETE'>".
                        "<input type='hidden' name='_token' value='{$token}'>". 
                        "<button type='submit' title = 'Delete' class='crm-action-delete text-muted' style='border:none; background:none; cursor:pointer;'><i class='fa fa-trash'></i></button>".
                    "</form>";
                $data[] = $nestedData;

            }
        }
          
        $json_data = array(
                    "draw"            => intval($request->input('draw')),  
                    "recordsTotal"    => intval($totalData),  
                    "recordsFiltered" => intval($totalFiltered), 
                    "data"            => $data   
                    );
            
        echo json_encode($json_data); 
        
    }    

    public function notifications()
    {
        return auth()->user()->Notifications()->get()->toArray();
    }   

    public function import() 
    {

        Excel::import(new UsersImport,request()->file('file'));
        return back();
    }   

    public function importExportView()
    {

       return view('admin.users.import');
    }    


}
