<?php

namespace App\Http\Controllers\Billboard;

use Spatie\Permission\Models\Role;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;


use Illuminate\Http\JsonResponse;
use Pion\Laravel\ChunkUpload\Exceptions\UploadFailedException;
use Illuminate\Http\UploadedFile;
use Pion\Laravel\ChunkUpload\Exceptions\UploadMissingFileException;
use Pion\Laravel\ChunkUpload\Handler\AbstractHandler;
use Pion\Laravel\ChunkUpload\Handler\HandlerFactory;
use Pion\Laravel\ChunkUpload\Receiver\FileReceiver;

use App\Models\Slider;
use App\Models\SliderTemp;

use DataTables;

class SliderController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    function __construct()
    {
         
         //$this->middleware('permission:Leaves-list|Leaves-show|Leaves-create|Leaves-edit|Leaves-delete|Leaves-import', ['only' => ['index','store']]);
         $this->middleware('permission:Permissions-list|Permissions-show', ['only' => ['index','show']]); 
         $this->middleware('permission:Permissions-list', ['only' => ['show']]);
         $this->middleware('permission:Permissions-create', ['only' => ['create','store']]);
         $this->middleware('permission:Permissions-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:Permissions-delete', ['only' => ['destroy']]);
         $this->middleware('permission:Permissions-import', ['only' => ['importExportView']]);
         $this->middleware('permission:Permissions-export', ['only' => ['export']]);
    }

    public function index(Request $request)
    {

         if(request()->ajax())
         {
               $data = DB::table('sliders')
                 ->select('*')
                 ->get();


             return datatables()->of($data)        
            ->addColumn('action', function ($row) {
                $show = route('billboard.sliders.show',$row->id);
                $edit =  route('billboard.sliders.edit',$row->id);
                $delete =  route('billboard.sliders.destroy',$row->id);
                $token =  csrf_token();                  
                //return "<a href='{$show}' class='btn btn-xs btn-primary'><i class='glyphicon glyphicon-edit'></i> Edit</a>";
                return "<div class='btn-group'>
                            <button aria-expanded='false' aria-haspopup='true' data-toggle='dropdown' data-boundary='viewport' class='btn-xs btn-primary dropdown-toggle' type='button'>
                            <small>Action</small> 
                            <span class='caret'></span></button>

                            <div class='dropdown-menu' style='font-size:12px'>
                              <a class='dropdown-item' href='{$show}'><i class='fa fa-desktop text-blue' style='color:blue;'></i><small> View</small></a>
                              <a class='dropdown-item' href='{$edit}'><i class='fa fa-edit text-green' style='color:green;'></i><small> Edit</small></a>
                              <form action='{$delete}' method='POST' onsubmit=\'return confirm('Are you sure you want to Delete Selected Record?');\' style='display: inline-block;'>".
                               "<input type='hidden' name='_method' value='DELETE'>".
                               "<input type='hidden' name='_token' value='{$token}'>". 
                               "<button type='submit' title = 'Delete' class='dropdown-item' style='border:none; background:none; cursor:pointer;'><i class='fa fa-trash text-red' style='color:red;'></i><small> Delete</small></button>".
                               "</form>
                            </div>
                        </div>";
                
            })
            ->make(true);


             // return datatables()->of($data)->make(true);
    


        }

         $sliders = DB::table('sliders')
             ->select('*')
              ->orderBy('id', 'DESC')
              ->get();

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('billboard.sliders.index', compact('sliders'));
         } else {
            return view('billboard.sliders.indexm', compact('sliders'));
         }              
              
         
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('billboard.sliders.create');
         } else {
            return view('billboard.sliders.createm');
         }         
        
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, Slider $sliders)
    {

        $userid = auth()->user()->id;   

        $sliders->title = $request->title; 
        $sliders->type = $request->type;

        if ($request->type == "Image"){
            if ($request->hasFile('filename1')){
                if($request->filename1->getClientOriginalName())
                {
                    $ext = $request->filename1->getClientOriginalExtension();
                    $filename1 = $request->filename1->getClientOriginalName();
                    $request->filename1->storeAs('public/banners', $filename1);
                }else{
                    $filename1='user.png';            
                }
            } else {
               $filename1='user.png';        
            }
            $sliders->filename = $filename1;    
            $sliders->duration = $request->duration;  

        } else {

            /*
            if ($request->hasFile('filename2')){
                if($request->filename2->getClientOriginalName())
                {
                    $ext = $request->filename2->getClientOriginalExtension();
                    $filename2 = $request->filename2->getClientOriginalName();

                    $request->filename2->storeAs('public/banners', $filename2);
                }else{
                    $filename2='user.png';            
                }
            } else {
               $filename2='user.png';        
            }
            */
            if(! empty($request->filename2)) {
                $sliders->filename = $request->filename2; 
                Storage::move('public/temp/'. $request->filename2, 'public/banners/'. $request->filename2);

                //$files = Storage::allFiles('public/temp');
                //Storage::delete($files);
                $slidertemps = DB::table('slider_temps')->where('userid','=', $userid)->get();

                foreach($slidertemps as $slidertemp1){
                    $file = public_path('storage/temp/' . $slidertemp1->filename);
                    if(file_exists($file)){
                      unlink('storage/temp/'.$slidertemp1->filename);
                    }
                }
                $slidertemps = DB::table('slider_temps')->where('userid','=', $userid)->delete();                    

               
            }
        }

        if ($request->validity == "Expiring"){
            $sliders->start = $request->start;
            $sliders->end = $request->end;
            $sliders->validity = $request->validity;
        } else {
            $sliders->validity = $request->validity;
        }
        $sliders->save();



        return redirect()->route('billboard.sliders.index'); 
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $sliders = DB::table('sliders')->where('id', $id)->first(); 

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('billboard.sliders.show', compact('sliders')); 
         } else {
            return view('billboard.sliders.showm', compact('sliders')); 
         } 

        
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $sliders = DB::table('sliders')->where('id', $id)->first(); 

         $agent = new \Jenssegers\Agent\Agent;    
         if($result = $agent->isDesktop()){
            return view('billboard.sliders.edit', compact('sliders')); 
         } else {
            return view('billboard.sliders.editm', compact('sliders')); 
         }         

        
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {

        $userid = auth()->user()->id; 
        $sliders = Slider::where('id', '=', $id)->first(); 

        $sliders->title = $request->title; 
        $sliders->type = $request->type;

        if ($request->type == "Image"){
            if ($request->hasFile('filename1')){
                $file = public_path('storage/banners/' . $sliders->filename);
                if(file_exists($file)){
                  unlink('storage/banners/'.$sliders->filename);
                } 
                if($request->filename1->getClientOriginalName())
                {
                    $ext = $request->filename1->getClientOriginalExtension();
                    $filename1 = $request->filename1->getClientOriginalName();
                    $request->filename1->storeAs('public/banners', $filename1);
                    $sliders->filename = $filename1;    
                  
           
                }
            }
    
        } else {
            /*
            if ($request->hasFile('filename2')){
                $file = public_path('storage/banners/' . $sliders->filename);
                if(file_exists($file)){
                  unlink('storage/banners/'.$sliders->filename);
                } 
                if($request->filename2->getClientOriginalName())
                {
                    $ext = $request->filename2->getClientOriginalExtension();
                    $filename2 = $request->filename2->getClientOriginalName();
                    $request->filename2->storeAs('public/banners', $filename2);
                    $sliders->filename = $filename2; 
         
                }
            }
            */
            if(! empty($request->filename2)) {
                unlink('storage/banners/'.$sliders->filename);
                $sliders->filename = $request->filename2; 
                Storage::move('public/temp/'. $request->filename2, 'public/banners/'. $request->filename2);

                $files = Storage::allFiles('public/temp');
                Storage::delete($files);

                $slidertemps = DB::table('slider_temps')->where('userid','=', $userid)->get();

                foreach($slidertemps as $slidertemp1){
                    $file = public_path('storage/temp/' . $slidertemp1->filename);
                    if(file_exists($file)){
                      unlink('storage/temp/'.$slidertemp1->filename);
                    }
                }
                $slidertemps = DB::table('slider_temps')->where('userid','=', $userid)->delete();    


                
               
            }            
        }

        if ($request->validity == "Expiring"){
            $sliders->start = $request->start;
            $sliders->end = $request->end;
            $sliders->validity = $request->validity;
        } else {
            $sliders->validity = $request->validity;
        }

        $sliders->duration = $request->duration;

        $sliders->save();

        return redirect()->route('billboard.sliders.index'); 
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function uploadLargeFiles(Request $request) {

        $userid = auth()->user()->id;

        $receiver = new FileReceiver('file', $request, HandlerFactory::classFromRequest($request));

        if (!$receiver->isUploaded()) {
            // file not uploaded
        }

        $fileReceived = $receiver->receive(); // receive file
        if ($fileReceived->isFinished()) { // file uploading is complete / all chunks are uploaded
            $file = $fileReceived->getFile(); // get file
            $extension = $file->getClientOriginalExtension();
            $fileName = str_replace('.'.$extension, '', $file->getClientOriginalName()); //file name without extenstion
            $fileName .= '_' . md5(time()) . '.' . $extension; // a unique file name

            $disk = Storage::disk(config('filesystems.default'));
            //$disk = Storage::local('public/banners/');
            //$path = Storage::disk('local')->put($fileName, $file);
            $path = $disk->putFileAs('public/temp', $file, $fileName);
            $path2 = $disk->putFileAs('storage/temp', $file, $fileName);

            $slidertemps = new SliderTemp;
            $slidertemps->userid = $userid;
            $slidertemps->filename = $fileName;
            $slidertemps->save();

            // delete chunked file
            unlink($file->getPathname());
            return [
                //'path' => asset('storage/banners/' . $path),
                'path' => asset($path2),
                'filename' => $fileName
            ];
        }

        // otherwise return percentage information
        $handler = $fileReceived->handler();
        return [
            'done' => $handler->getPercentageDone(),
            'status' => true
        ];
    } 
}
